@@ -312,7 +312,7 @@ static void mod_mapcache_child_init(apr_pool_t *pool, server_rec *s)
     int i,rv;
     for(i=0;i<cfg->aliases->nelts;i++) {
       mapcache_alias_entry *alias_entry = APR_ARRAY_IDX(cfg->aliases,i,mapcache_alias_entry*);
-      rv = mapcache_connection_pool_create(&(alias_entry->cp),pool);
+      rv = mapcache_connection_pool_create(alias_entry->cfg, &(alias_entry->cp),pool);
       ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s, "creating a child process mapcache connection pool on server %s for alias %s", s->server_hostname, alias_entry->endpoint);
       if(rv!=APR_SUCCESS) {
         ap_log_error(APLOG_MARK, APLOG_CRIT, 0, s, "failed to create mapcache connection pool");
@@ -320,7 +320,7 @@ static void mod_mapcache_child_init(apr_pool_t *pool, server_rec *s)
     }
     for(i=0;i<cfg->quickaliases->nelts;i++) {
       mapcache_alias_entry *alias_entry = APR_ARRAY_IDX(cfg->quickaliases,i,mapcache_alias_entry*);
-      rv = mapcache_connection_pool_create(&(alias_entry->cp),pool);
+      rv = mapcache_connection_pool_create(alias_entry->cfg, &(alias_entry->cp),pool);
       ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s, "creating a child process mapcache connection pool on server %s for alias %s", s->server_hostname, alias_entry->endpoint);
       if(rv!=APR_SUCCESS) {
         ap_log_error(APLOG_MARK, APLOG_CRIT, 0, s, "failed to create mapcache connection pool");
@@ -697,6 +697,10 @@ static const command_rec mod_mapcache_cmds[] = {
   { NULL }
 } ;
 
+#ifdef APLOG_USE_MODULE
+APLOG_USE_MODULE(mapcache);
+#endif
+
 module AP_MODULE_DECLARE_DATA mapcache_module = {
   STANDARD20_MODULE_STUFF,
   NULL,
