@@ -105,6 +105,8 @@ typedef struct mapcache_image mapcache_image;
 typedef struct mapcache_grid mapcache_grid;
 typedef struct mapcache_grid_level mapcache_grid_level;
 typedef struct mapcache_grid_link mapcache_grid_link;
+typedef struct mapcache_rule mapcache_rule;
+typedef struct mapcache_ruleset mapcache_ruleset;
 typedef struct mapcache_context mapcache_context;
 typedef struct mapcache_dimension mapcache_dimension;
 typedef struct mapcache_requested_dimension mapcache_requested_dimension;
@@ -839,6 +841,11 @@ struct mapcache_cfg {
    */
   apr_hash_t *grids;
 
+  /**
+   * hashtable containing (pre)defined rulesets
+   */
+  apr_hash_t *rulesets;
+
   /**
    * the format to use for some miscelaneaous operations:
    *  - creating an empty image
@@ -875,6 +882,12 @@ struct mapcache_cfg {
   /* return 404 on potentially blocking operations (proxying, source getmaps,
    locks on metatile waiting, ... Used for nginx module */
   int non_blocking;
+
+  // Parameters for connection_pool:
+  // - cp_hmax defines the maximum number of open connections at the same time
+  // - cp_ttl defines the maximum amount of time in microseconds an unused connection is valid
+  int cp_hmax;
+  int cp_ttl;
 };
 
 /**
@@ -893,9 +906,11 @@ MS_DLL_EXPORT mapcache_cache* mapcache_configuration_get_cache(mapcache_cfg *con
 mapcache_grid *mapcache_configuration_get_grid(mapcache_cfg *config, const char *key);
 MS_DLL_EXPORT mapcache_tileset* mapcache_configuration_get_tileset(mapcache_cfg *config, const char *key);
 mapcache_image_format *mapcache_configuration_get_image_format(mapcache_cfg *config, const char *key);
+mapcache_ruleset *mapcache_configuration_get_ruleset(mapcache_cfg *config, const char *key);
 void mapcache_configuration_add_image_format(mapcache_cfg *config, mapcache_image_format *format, const char * key);
 void mapcache_configuration_add_source(mapcache_cfg *config, mapcache_source *source, const char * key);
 void mapcache_configuration_add_grid(mapcache_cfg *config, mapcache_grid *grid, const char * key);
+void mapcache_configuration_add_ruleset(mapcache_cfg *config, mapcache_ruleset *ruleset, const char * key);
 void mapcache_configuration_add_tileset(mapcache_cfg *config, mapcache_tileset *tileset, const char * key);
 void mapcache_configuration_add_cache(mapcache_cfg *config, mapcache_cache *cache, const char * key);
 
@@ -1057,6 +1072,12 @@ struct mapcache_grid_link {
   mapcache_extent_i *grid_limits;
   int minz,maxz;
 
+  /**
+   * rules (mapcache_rule) for each zoom level
+   * index in array = zoom level
+   */
+  apr_array_header_t *rules;
+
   /**
    * tiles above this zoom level will not be stored to the cache, but will be
    * dynamically generated (either by reconstructing from lower level tiles, or
@@ -1069,6 +1090,46 @@ struct mapcache_grid_link {
   apr_array_header_t *intermediate_grids;
 };
 
+/**\class mapcache_rule
+ * \brief a zoom level rule
+ */
+struct mapcache_rule {
+  /**
+   * rule for zoom level
+   */
+  int zoom_level;
+  /**
+   * color of tiles when outside visible extent, ARGB
+   */
+  unsigned int hidden_color;
+  /**
+   * tile to return when outside visible extent
+   */
+  mapcache_buffer *hidden_tile;
+  /**
+   * visible extents, array of mapcache_extent
+   */
+  apr_array_header_t *visible_extents;
+  /**
+   * visible limits, array of mapcache_extent_i
+   */
+  apr_array_header_t *visible_limits;
+};
+
+/**\class mapcache_ruleset
+ * \brief a set of rules
+ */
+struct mapcache_ruleset {
+  /**
+   * the name of this ruleset
+   */
+  char *name;
+  /**
+   * rules (mapcache_rule)
+   */
+  apr_array_header_t *rules;
+};
+
 /**\class mapcache_tileset
  * \brief a set of tiles that can be requested by a client, created from a mapcache_source
  *        stored by a mapcache_cache in a mapcache_format
@@ -1142,6 +1203,12 @@ struct mapcache_tileset {
 
   mapcache_dimension_assembly_type dimension_assembly_type;
 
+  /**
+   * Maximum zoom level for activating multithreaded subtile retrieval
+   * -1 means 'not activated'
+   */
+  int assembly_threaded_fetching_maxzoom;
+
   /**
    * image to be used as a watermark
    */
@@ -1163,7 +1230,8 @@ void mapcache_tileset_get_map_tiles(mapcache_context *ctx, mapcache_tileset *til
                                     mapcache_extent *bbox, int width, int height,
                                     int *ntiles,
                                     mapcache_tile ***tiles,
-                                    mapcache_grid_link **effectively_used_grid_link);
+                                    mapcache_grid_link **effectively_used_grid_link,
+                                    apr_array_header_t *dimensions);
 
 mapcache_image* mapcache_tileset_assemble_map_tiles(mapcache_context *ctx, mapcache_tileset *tileset,
     mapcache_grid_link *grid_link,
@@ -1278,6 +1346,49 @@ MS_DLL_EXPORT mapcache_http_response* mapcache_core_proxy_request(mapcache_conte
 MS_DLL_EXPORT mapcache_http_response* mapcache_core_respond_to_error(mapcache_context *ctx);
 
 
+/* in ruleset.c */
+
+/**
+ * \brief allocate and initialize a new ruleset
+ * @param pool
+ */
+mapcache_ruleset* mapcache_ruleset_create(apr_pool_t *pool);
+
+/**
+ * \brief allocate and initialize a new rule
+ * @param pool
+ */
+mapcache_rule* mapcache_ruleset_rule_create(apr_pool_t *pool);
+
+/**
+ * \brief clone a rule
+ * @param pool
+ * @param rule
+ */
+mapcache_rule* mapcache_ruleset_rule_clone(apr_pool_t *pool, mapcache_rule *rule);
+
+/**
+ * \brief get rule for zoom level, or NULL if none exist
+ * @param ruleset
+ * @param zoom_level
+ */
+mapcache_rule* mapcache_ruleset_rule_find(apr_array_header_t *rules, int zoom_level);
+
+/**
+ * \brief get rule at index, or NULL if none exist
+ * @param rules
+ * @param idx
+ */
+mapcache_rule* mapcache_ruleset_rule_get(apr_array_header_t *rules, int idx);
+
+/**
+ * \brief check if tile is within visible extent
+ * @param rule
+ * @param tile
+ */
+int mapcache_ruleset_is_visible_tile(mapcache_rule* rule, mapcache_tile *tile);
+
+
 /* in grid.c */
 mapcache_grid* mapcache_grid_create(apr_pool_t *pool);
 
@@ -1320,6 +1431,7 @@ int mapcache_grid_get_level(mapcache_context *ctx, mapcache_grid *grid, double *
  * \param tolerance the number of tiles around the given extent that can be requested without returning an error.
  */
 MS_DLL_EXPORT void mapcache_grid_compute_limits(const mapcache_grid *grid, const mapcache_extent *extent, mapcache_extent_i *limits, int tolerance);
+void mapcache_grid_compute_limits_at_level(const mapcache_grid *grid, const mapcache_extent *extent, mapcache_extent_i *limits_ptr, int tolerance, int zoom_level);
 
 /* in util.c */
 MS_DLL_EXPORT int mapcache_util_extract_int_list(mapcache_context *ctx, const char* args, const char *sep, int **numbers,
@@ -1534,6 +1646,11 @@ void _mapcache_imageio_jpeg_decode_to_image(mapcache_context *ctx, mapcache_buff
  */
 mapcache_image_format_type mapcache_imageio_header_sniff(mapcache_context *ctx, mapcache_buffer *buffer);
 
+/**
+ * \brief lookup the first few bytes of a buffer to check for alpha channel
+ */
+mapcache_image_alpha_type mapcache_imageio_alpha_sniff(mapcache_context *ctx, mapcache_buffer *buffer);
+
 /**
  * \brief checks if the given buffer is a recognized image format
  */
@@ -1563,6 +1680,7 @@ struct mapcache_requested_dimension {
   mapcache_dimension *dimension;
   char *requested_value;
   char *cached_value;
+  apr_array_header_t *cached_entries_for_value;
 };
 
 void mapcache_tile_set_cached_dimension(mapcache_context *ctx, mapcache_tile *tile, const char *name, const char *value);
@@ -1575,7 +1693,9 @@ MS_DLL_EXPORT apr_array_header_t *mapcache_requested_dimensions_clone(apr_pool_t
 
 struct mapcache_dimension {
   mapcache_dimension_type type;
+  const char *class_name;
   int isTime;
+  int wms_querybymap_minzoom;
   char *name;
   char *unit;
   apr_table_t *metadata;
@@ -1638,7 +1758,7 @@ struct mapcache_pooled_connection {
 typedef void (*mapcache_connection_constructor)(mapcache_context *ctx, void **connection, void *params);
 typedef void (*mapcache_connection_destructor)(void *connection);
 
-MS_DLL_EXPORT apr_status_t mapcache_connection_pool_create(mapcache_connection_pool **cp, apr_pool_t *server_pool);
+MS_DLL_EXPORT apr_status_t mapcache_connection_pool_create(mapcache_cfg *cfg, mapcache_connection_pool **cp, apr_pool_t *server_pool);
 mapcache_pooled_connection* mapcache_connection_pool_get_connection(mapcache_context *ctx, char *key,
         mapcache_connection_constructor constructor,
         mapcache_connection_destructor destructor,
