@@ -33,6 +33,7 @@
 #include <apr_tables.h>
 #include <curl/curl.h>
 #include <math.h>
+#include <float.h>
 #include <apr_file_io.h>
 
 #ifndef _WIN32
@@ -171,6 +172,11 @@ char *mapcache_util_str_replace(apr_pool_t *pool, const char *string, const char
   return newstr;
 }
 
+char *mapcache_util_dbl_replace(apr_pool_t *pool, const char *string, const char *substr, double replacement)
+{
+  return mapcache_util_str_replace(pool,string,substr,apr_psprintf(pool,"%.*e",DBL_DIG,replacement));
+}
+
 char* mapcache_util_str_sanitize(apr_pool_t *pool, const char *str, const char* from, char to)
 {
   char *pstr = apr_pstrdup(pool,str);
@@ -185,6 +191,37 @@ char* mapcache_util_str_sanitize(apr_pool_t *pool, const char *str, const char*
   return pstr;
 }
 
+char * mapcache_util_str_replace_all(apr_pool_t *pool, const char *string, const char *substr, const char *replacement)
+{
+  if (!replacement) {
+    return apr_pstrdup(pool,string);
+  } else {
+    int lenstring = strlen(string);
+    int lensubstr = strlen(substr);
+    int lenreplacement = strlen(replacement);
+    // Provision is made for worst case scenario, i.e., maximum possible number of substring occurrences in string
+    int lennewstr = (lenreplacement>lensubstr) ? lenreplacement*(1+lenstring/lensubstr) : lenstring;
+    char * newstr = apr_pcalloc(pool, lennewstr+1);
+    const char * xin = string;
+    char * xout = newstr;
+    char * match;
+    while ((match = strstr(xin,substr))) {
+      memcpy(xout, xin, match - xin);
+      xout += match - xin;
+      memcpy(xout, replacement, lenreplacement);
+      xout += lenreplacement;
+      xin = match + lensubstr;
+    }
+    strcpy(xout,xin);
+    return newstr;
+  }
+}
+
+char * mapcache_util_dbl_replace_all(apr_pool_t *pool, const char *string, const char *substr, double replacement)
+{
+  return mapcache_util_str_replace_all(pool,string,substr,apr_psprintf(pool,"%.*e",DBL_DIG,replacement));
+}
+
 char* mapcache_util_str_xml_escape(apr_pool_t *pool, const char *str,
                                    mapcache_util_xml_section_type xml_section_type)
 {
